@@ -1,5 +1,6 @@
 ###
 # Copyright (c) 2002-2005, Jeremiah Fincher
+# Copyright (c) 2008-2009, James Vega
 # All rights reserved.
 #
 # Redistribution and use in source and binary forms, with or without
@@ -163,6 +164,8 @@ def reply(msg, s, prefixNick=None, private=None,
     # Ok, let's make the target:
     # XXX This isn't entirely right.  Consider to=#foo, private=True.
     target = ircutils.replyTo(msg)
+    if ircutils.isChannel(to):
+        target = to
     if ircutils.isChannel(target):
         channel = target
     else:
@@ -183,6 +186,8 @@ def reply(msg, s, prefixNick=None, private=None,
             target = msg.nick
         else:
             target = to
+    if action:
+        prefixNick = False
     if to is None:
         to = msg.nick
     # Ok, now let's make the payload:
@@ -227,10 +232,13 @@ def getHelp(method, name=None, doc=None):
         s = '(%s) -- %s' % (ircutils.bold(s), help)
     return utils.str.normalizeWhitespace(s)
 
-def getSyntax(method, name=None):
+def getSyntax(method, name=None, doc=None):
     if name is None:
         name = method.__name__
-    doclines = method.__doc__.splitlines()
+    if doc is None:
+        doclines = method.__doc__.splitlines()
+    else:
+        doclines = doc.splitlines()
     return '%s %s' % (name, doclines[0])
 
 class Error(Exception):
@@ -347,17 +355,6 @@ def tokenize(s, channel=None):
     except ValueError, e:
         raise SyntaxError, str(e)
 
-def formatArgumentError(method, name=None):
-    if name is None:
-        name = method.__name__
-    if hasattr(method, '__doc__') and method.__doc__:
-        if conf.get(conf.supybot.reply.showSimpleSyntax, dynamic.channel):
-            return getSyntax(method, name=name)
-        else:
-            return getHelp(method, name=name)
-    else:
-        return 'Invalid arguments for %s.' % method.__name__
-
 def formatCommand(command):
     return ' '.join(command)
 
@@ -815,7 +812,8 @@ class NestedCommandsIrcProxy(ReplyIrcProxy):
             self.prefixNick = prefixNick
         if action is not None:
             self.action = self.action or action
-            self.prefixNick = False
+            if action:
+                self.prefixNick = False
         if notice is not None:
             self.notice = self.notice or notice
         if private is not None:
@@ -1051,7 +1049,7 @@ class BasePlugin(object):
                 setattr(self, attr, cb)
                 self.cbs.append(cb)
                 cb.log = log.getPluginLogger('%s.%s' % (self.name(),cb.name()))
-        super(BasePlugin, self).__init__(*args, **kwargs)
+        super(BasePlugin, self).__init__()
 
 class SynchronizedAndFirewalled(log.MetaFirewall, utils.python.Synchronized):
     pass # Necessary for the metaclass compatibility issue.
@@ -1139,22 +1137,29 @@ class Commands(BasePlugin):
             assert command[0] == self.canonicalName()
             return self.getCommandMethod(command[1:])
         else:
-            return getattr(self, command[0])
+            method = getattr(self, command[0])
+            if inspect.ismethod(method):
+                code = method.im_func.func_code
+                if inspect.getargs(code)[0] == self.commandArgs:
+                    return method
+                else:
+                    raise AttributeError
 
-    def listCommands(self):
-        commands = []
+    def listCommands(self, pluginCommands=[]):
+        commands = set(pluginCommands)
         for s in dir(self):
-            if self.isCommand(s):
-                commands.append(s)
+            if self.isCommandMethod(s):
+                commands.add(s)
         for cb in self.cbs:
             name = cb.canonicalName()
             for command in cb.listCommands():
                 if command == name:
-                    commands.append(command)
+                    commands.add(command)
                 else:
-                    commands.append(' '.join([name, command]))
-        commands.sort()
-        return commands
+                    commands.add(' '.join([name, command]))
+        L = list(commands)
+        L.sort()
+        return L
 
     def callCommand(self, command, irc, msg, *args, **kwargs):
         method = self.getCommandMethod(command)
@@ -1178,8 +1183,11 @@ class Commands(BasePlugin):
         except (getopt.GetoptError, ArgumentError), e:
             self.log.debug('Got %s, giving argument error.',
                            utils.exnToString(e))
-            method = self.getCommandMethod(command)
-            irc.reply(formatArgumentError(method, name=formatCommand(command)))
+            help = self.getCommandHelp(command)
+            if help.endswith('command has no help.'):
+                irc.error('Invalid arguments for %s.' % method.__name__)
+            else:
+                irc.reply(help)
         except (SyntaxError, Error), e:
             self.log.debug('Error return: %s', utils.exnToString(e))
             irc.error(str(e))
@@ -1190,10 +1198,18 @@ class Commands(BasePlugin):
             else:
                 irc.replyError()
 
-    def getCommandHelp(self, command):
+    def getCommandHelp(self, command, simpleSyntax=None):
         method = self.getCommandMethod(command)
+        help = getHelp
+        chan = None
+        if dynamic.msg is not None:
+            chan = dynamic.msg.args[0]
+        if simpleSyntax is None:
+            simpleSyntax = conf.get(conf.supybot.reply.showSimpleSyntax, chan)
+        if simpleSyntax:
+            help = getSyntax
         if hasattr(method, '__doc__'):
-            return getHelp(method, name=formatCommand(command))
+            return help(method, name=formatCommand(command))
         else:
             return format('The %q command has no help.',formatCommand(command))
 
@@ -1220,7 +1236,9 @@ class PluginMixin(BasePlugin, irclib.IrcCallback):
     def __call__(self, irc, msg):
         irc = SimpleProxy(irc, msg)
         if msg.command == 'PRIVMSG':
-            if self.noIgnore or not ircdb.checkIgnored(msg.prefix,msg.args[0]):
+            if self.noIgnore or \
+               not ircdb.checkIgnored(msg.prefix,msg.args[0]) or \
+               not ircutils.isUserHostmask(msg.prefix):  # Some services impl.
                 self.__parent.__call__(irc, msg)
         else:
             self.__parent.__call__(irc, msg)
@@ -1297,8 +1315,10 @@ Privmsg = Plugin # Backwards compatibility.
 
 class PluginRegexp(Plugin):
     """Same as Plugin, except allows the user to also include regexp-based
-    callbacks.  All regexp-based callbacks must be specified in a set (or
-    list) attribute "regexps".
+    callbacks.  All regexp-based callbacks must be specified in the set (or
+    list) attribute "regexps", "addressedRegexps", or "unaddressedRegexps"
+    depending on whether they should always be triggered, triggered only when
+    the bot is addressed, or triggered only when the bot isn't addressed.
     """
     flags = re.I
     # 'regexps' methods are called whether the message is addressed or not.
