@@ -1,5 +1,6 @@
 ###
 # Copyright (c) 2002-2005, Jeremiah Fincher
+# Copyright (c) 2009, James Vega
 # All rights reserved.
 #
 # Redistribution and use in source and binary forms, with or without
@@ -249,6 +250,8 @@ def getNetworkIrc(irc, msg, args, state, errorIfNoMatch=False):
         state.args.append(irc)
 
 def getHaveOp(irc, msg, args, state, action='do that'):
+    if not state.channel:
+        getChannel(irc, msg, args, state)
     if state.channel not in irc.state.channels:
         state.error('I\'m not even in %s.' % state.channel, Raise=True)
     if not irc.state.channels[state.channel].isOp(irc.nick):
@@ -273,8 +276,10 @@ def getHostmask(irc, msg, args, state):
 
 def getBanmask(irc, msg, args, state):
     getHostmask(irc, msg, args, state)
-    # XXX Channel-specific stuff.
-    state.args[-1] = ircutils.banmask(state.args[-1])
+    if not state.channel:
+        getChannel(irc, msg, args, state)
+    banmaskstyle = conf.supybot.protocols.irc.banmask
+    state.args[-1] = banmaskstyle.makeBanmask(state.args[-1])
 
 def getUser(irc, msg, args, state):
     try:
@@ -496,6 +501,8 @@ def getEmail(irc, msg, args, state):
 def getHttpUrl(irc, msg, args, state):
     if utils.web.httpUrlRe.match(args[0]):
         state.args.append(args.pop(0))
+    elif utils.web.httpUrlRe.match('http://' + args[0]):
+        state.args.append('http://' + args.pop(0))
     else:
         state.errorInvalid('http url', args[0])
 
