@@ -1,5 +1,6 @@
 ###
 # Copyright (c) 2002-2005, Jeremiah Fincher
+# Copyright (c) 2009, James Vega
 # All rights reserved.
 #
 # Redistribution and use in source and binary forms, with or without
@@ -73,6 +74,10 @@ addConverter('factoid', getFactoid)
 addConverter('factoidId', getFactoidId)
 
 class Factoids(callbacks.Plugin, plugins.ChannelDBHandler):
+    def __init__(self, irc):
+        callbacks.Plugin.__init__(self, irc)
+        plugins.ChannelDBHandler.__init__(self)
+
     def makeDb(self, filename):
         if os.path.exists(filename):
             return sqlite.connect(filename)
@@ -99,14 +104,23 @@ class Factoids(callbacks.Plugin, plugins.ChannelDBHandler):
         db.commit()
         return db
 
-    def getCommandHelp(self, command):
+    def getCommandHelp(self, command, simpleSyntax=None):
         method = self.getCommandMethod(command)
         if method.im_func.func_name == 'learn':
-            s = self.registryValue('learnSeparator', dynamic.msg.args[0])
-            return callbacks.getHelp(method,
-                                     doc=method._fake__doc__ % (s, s),
-                                     name=callbacks.formatCommand(command))
-        return super(Factoids, self).getCommandHelp(self, command)
+            chan = None
+            if dynamic.msg is not None:
+                chan = dynamic.msg.args[0]
+            s = self.registryValue('learnSeparator', chan)
+            help = callbacks.getHelp
+            if simpleSyntax is None:
+                simpleSyntax = conf.get(conf.supybot.reply.showSimpleSyntax,
+                                        chan)
+            if simpleSyntax:
+                help = callbacks.getSyntax
+            return help(method,
+                        doc=method._fake__doc__ % (s, s),
+                        name=callbacks.formatCommand(command))
+        return super(Factoids, self).getCommandHelp(command, simpleSyntax)
 
     def learn(self, irc, msg, args, channel, key, factoid):
         db = self.getDb(channel)
